@@ -148,7 +148,8 @@ class PythonExpression:
         ast.USub
     }
 
-    def __init__(self, definition, *variables, **constants):
+    def __init__(self, definition, *variables, constants=None, all_variables_allowed=False):
+        constants = constants or {}
         variables = set(variables) | set(constants)
         try:
             expression = ast.parse(definition)
@@ -167,7 +168,7 @@ class PythonExpression:
                 'the following operations are not allowed: {}'.format(
                     ', '.join(t.__name__ for t in types - self._ALLOWED_NODE_TYPES))
             )
-        if not names <= variables:
+        if not all_variables_allowed and not names <= variables:
             raise VariablesMissing(
                 'the following variable names are unknown: {}'.format(
                     ', '.join(names - variables))
@@ -617,18 +618,21 @@ class DelayedNumSeqArgument(DelayedArgument):
     def ev(self, expression: Union[str, ByteString]) -> Iterable[int]:
         """
         Final modifier `e:expression` or `eval:expression`; uses a `refinery.lib.argformats.PythonExpression`
-        parser to process expressions that may contain the variable `N` whose value will be the size of the
-        input data.
+        parser to process expressions. The expression can contain any meta variable that is attached to the
+        chunk. The `refinery.trivia` unit can be used to attach information such as chunk size and the chunk
+        index within the current frame (see `refinery.lib.frame`).
         """
         if not isinstance(expression, str):
             try:
                 expression = expression.decode('ascii')
             except AttributeError:
                 return expression
-        ev = PythonExpression(expression, 'N')
-        if 'N' in ev.variables:
-            return lambda d: self._iter(ev(N=len(d)))
-        return self._iter(ev())
+        evaluator = PythonExpression(expression, all_variables_allowed=True)
+        if evaluator.variables:
+            def finalize(chunk):
+                return self._iter(evaluator(**chunk.meta))
+            return finalize
+        return self._iter(evaluator())
 
     @handler.register('unpack', final=True)
     def unpack(self, expression: str, size=None) -> Iterable[int]:
