@@ -5,6 +5,7 @@ Implements several popular block and stream ciphers.
 """
 from typing import Iterable, Any, ByteString
 from Crypto.Util.Padding import pad, unpad
+from Crypto.Util import Counter
 
 from ... import arg, Unit, Executable, RefineryCriticalException, RefineryPartialResult
 from ....lib.argformats import OptionFactory, extract_options
@@ -143,8 +144,11 @@ class StandardCipherExecutable(CipherExecutable):
                 )
             cls._bcmspec_ = OptionFactory(modes, ignorecase=True)
             cls._argspec_['mode'].merge_all(arg(
-                dest='mode', type=str.upper, metavar='mode', choices=list(modes),
-                help='Choose cipher mode to be used. Possible values are: {}.'.format(', '.join(modes))
+                '-M', '--mode', type=str.upper, metavar='MODE', nargs=arg.delete, choices=list(modes),
+                help=(
+                    'Choose cipher mode to be used. Possible values are: {}. By default, the CBC mode'
+                    '  is used when an IV is is provided, and ECB otherwise.'.format(', '.join(modes))
+                )
             ))
 
 
@@ -166,31 +170,60 @@ class StandardCipherUnit(CipherUnit, metaclass=StandardCipherExecutable):
 
 class StandardBlockCipherUnit(BlockCipherUnitBase, StandardCipherUnit):
 
-    def __init__(self, mode, key, iv=B'', padding=None):
-        super().__init__(key=key, iv=iv, padding=padding, mode=self._bcmspec_(mode))
+    def __init__(self, key, iv=B'', padding=None, mode=None):
+        mode = self._bcmspec_(mode or iv and 'CBC' or 'ECB')
+        if iv:
+            if mode.name == 'ECB':
+                raise ValueError('No initialization vector can be specified for ECB mode.')
+            bounds = {
+                'CCM': (7, 14),
+                'OCB': (1, 16),
+                'CTR': (1, 17),
+            }.get(mode.name, None)
+            if bounds and len(iv) not in range(*bounds):
+                raise ValueError(F'Invalid nonce length, must be in {bounds} for {mode.name}.')
+        super().__init__(key=key, iv=iv, padding=padding, mode=mode)
 
     def _get_cipher_instance(self, **optionals) -> Any:
-        if not self.args.mode.name == 'ECB':
-            optionals['IV'] = self.iv
+        mode = self.args.mode.name
+        if mode != 'ECB':
+            if mode == 'CTR' and len(self.iv) == 16:
+                counter = Counter.new(self.blocksize * 8,
+                    initial_value=int.from_bytes(self.iv, 'big'))
+                optionals['counter'] = counter
+            elif mode in ('CCM', 'EAX', 'GCM', 'SIV', 'OCB', 'CTR'):
+                optionals['nonce'] = self.iv
+            elif mode in ('CBC', 'CFB', 'OFB', 'OPENPGP'):
+                optionals['iv'] = self.iv
             self.log_info('initial vector:', self.iv.hex())
         if self.args.mode:
             optionals['mode'] = self.args.mode.value
         try:
             return super()._get_cipher_instance(**optionals)
         except TypeError:
-            if 'IV' not in optionals:
+            if 'iv' not in optionals:
                 raise
-            del optionals['IV']
+            del optionals['iv']
             if self.iv:
-                self.log_info('ignoring IV for mode', self.args.mode)
+                self.log_info('ignoring iv for mode', self.args.mode)
             return self._stdcipher_.new(key=self.args.key, **optionals)
 
 
-class LatinStreamCipher(StandardCipherUnit):
+class LatinCipherUnit(StreamCipherUnit, abstract=True):
+    key_sizes = 16, 32
 
-    def __init__(self, key, nonce:
-        arg('-N', help='Specify the one-time use nonce; the default value is "REFINERY".') = B'REFINERY'
+    def __init__(
+        self, key,
+        nonce: arg(help='The nonce. Default is the string {default}.') = B'REFINERY',
+        magic: arg('-m', help='The magic constant; depends on the key size by default.') = B'',
+        offset: arg.number('-x', help='Optionally specify the stream index, default is {default}.') = 0,
+        rounds: arg.number('-r', help='The number of rounds. Has to be an even number.') = 20,
     ):
+        super().__init__(key=key, nonce=nonce, magic=magic, offset=offset, rounds=rounds)
+
+
+class LatinCipherStandardUnit(StandardCipherUnit):
+    def __init__(self, key, nonce: arg(help='The nonce. Default is the string {default}.') = B'REFINERY'):
         super().__init__(key, nonce=nonce)
 
     def _get_cipher_instance(self, **optionals) -> Any:
