@@ -39,10 +39,17 @@ class esc(Unit):
     def __init__(self,
         hex     : arg.switch('-x', help='Hex encode everything, do not use C escape sequences.') = False,
         unicode : arg.switch('-u', help='Use unicode escape sequences and UTF-8 encoding.') = False,
-        greedy  : arg.switch('-g', help='Replace \\x by x and \\u by u when not followed by two or four hex digits, respectively.') = False
+        greedy  : arg.switch('-g', help='Replace \\x by x and \\u by u when not followed by two or four hex digits, respectively.') = False,
+        quoted  : arg.switch('-q', help='Remove enclosing quotes while decoding and add them for encoding.') = False
     ) -> Unit: pass  # noqa
 
     def process(self, data):
+        if self.args.quoted:
+            quote = data[0]
+            if data[~0] != quote:
+                raise ValueError('string is not correctly quoted')
+            data = data[1:~0]
+
         if self.args.unicode:
             return data.decode('UNICODE_ESCAPE').encode(self.codec)
 
@@ -62,14 +69,19 @@ class esc(Unit):
 
     def reverse(self, data):
         if self.args.unicode:
-            return data.decode(self.codec).encode('UNICODE_ESCAPE')
-        elif not self.args.hex:
-            def escape(match):
-                c = match[0][0]
-                return self._ESCAPE.get(c, RB'\x%02x' % c)
+            string = data.decode(self.codec).encode('UNICODE_ESCAPE')
         else:
-            def escape(match):
-                c = match[0][0]
-                return RB'\x%02x' % c
-
-        return re.sub(RB'[\x00-\x1F\x22\x27\x5C\x7F-\xFF]', escape, data)
+            if not self.args.hex:
+                def escape(match):
+                    c = match[0][0]
+                    return self._ESCAPE.get(c, RB'\x%02x' % c)
+                string = re.sub(RB'[\x00-\x1F\x22\x27\x5C\x7F-\xFF]', escape, data)
+            else:
+                string = bytearray(4 * len(data))
+                for k in range(len(data)):
+                    a = k * 4
+                    b = k * 4 + 4
+                    string[a:b] = RB'\x%02x' % data[k]
+        if self.args.quoted:
+            string = B'"%s"' % string
+        return string
