@@ -29,9 +29,10 @@ class UnpackResult:
             self.data = self.data()
         return self.data
 
-    def __init__(self, path: str, data: Union[ByteString, Callable[[], ByteString]]):
+    def __init__(self, path: str, data: Union[ByteString, Callable[[], ByteString]], **meta):
         self.path = path
         self.data = data
+        self.meta = meta
 
 
 class EndOfStringNotFound(ValueError):
@@ -48,7 +49,7 @@ class PathPattern:
         elif not regex:
             if fuzzy and not pp.startswith('*') and not pp.endswith('*'):
                 pp = F'*{pp}*'
-            self.stops = [pp[:k] for k, c in enumerate(pp) if c in '/*?']
+            self.stops = [stop for stop in re.split(R'(.*?[/*?])', pp) if stop]
             pp = fnmatch.translate(pp)
         self.pattern = re.compile(pp)
 
@@ -79,6 +80,8 @@ class PathExtractorUnit(Unit, abstract=True):
         regex: arg.switch('-r', help='Use regular expressions instead of wildcard patterns.') = False,
         fuzzy: arg.switch('-z', help='Wrap wildcard expressions in asterixes automatically '
             '(no effect on regular expressions).') = False,
+        meta: arg('-m', metavar='NAME',
+            help='Name of the meta variable to receive the extracted path. The default value is "{default}".') = b'path',
         **keywords
     ):
         paths = paths or ['*']
@@ -89,6 +92,7 @@ class PathExtractorUnit(Unit, abstract=True):
             ],
             list=list,
             join=join,
+            meta=meta,
             **keywords
         )
 
@@ -121,22 +125,24 @@ class PathExtractorUnit(Unit, abstract=True):
         raise NotImplementedError
 
     def process(self, data: ByteString) -> ByteString:
-        try:
-            root = data['path']
-        except KeyError:
-            root = ''
-
         results = []
+        metavar = self.args.meta.decode(self.codec)
         paths = collections.defaultdict(set)
         self.__unknown = 0
 
+        try:
+            root = data[metavar]
+        except KeyError:
+            root = ''
+
         for result in self.unpack(data):
             if self._check_path(result): results.append(result)
 
         for p in self.args.patterns:
             for result in results:
                 path = result.path
-                path = '/'.join(path.split('\\'))
+                if '\\' in path:
+                    path = '/'.join(path.split('\\'))
                 if not p.check(path):
                     continue
                 if not self.args.list:
@@ -155,4 +161,5 @@ class PathExtractorUnit(Unit, abstract=True):
                     continue
                 else:
                     self.log_info(path)
-                yield self.labelled(result.get_data(), path=path)
+                result.meta[metavar] = path
+                yield self.labelled(result.get_data(), **result.meta)
