@@ -10,7 +10,7 @@ class dnmr(PathExtractorUnit):
     Extracts subfiles from .NET managed resources.
     """
     def __init__(
-        self, *paths, list=False, join=False,
+        self, *paths, list=False, join=False, meta=b'name',
         raw: arg.switch('-r', help='Do not deserialize the managed resource entry data.') = False
     ):
         super().__init__(*paths, list=list, join=join, raw=raw)
